from vpython import *
#GlowScript 3.0 VPython
# A BBH 

scene = display(title='BBH (500,300 M_sun) coalescing on the hour. RH mouse rotates, LH+RH zooms. Change masses with a/z s/x,  t toggles orbital plane. Adjust time with arrow up/down', width=1890, height=1050)

## set up the masses. 
m1 = sphere(color=color.white)
m2 = sphere(color=color.white)
plane = box(pos = vector(0,0,0), size = vector(30000,30000,100), opacity=0.0) #reference plane. Toggle with "t"
G = 6.67408e-11
c = 299792458.0
M_sun = 1.9884754153381438e+30
pi = 3.141592654
r_s = 2*G*M_sun/c**2

def keyInput(evt):
    global tc
    s = evt.key
    if (s == 'a'): 
        m1.mass= m1.mass + 10
        m1.radius = m1.mass
    if (s == 'z'): 
        m1.mass= m1.mass - 10
        m1.radius = m1.mass
    if (s == 's'): 
        m2.mass= m2.mass + 10
        m2.radius = m2.mass
    if (s == 'x'): 
        m2.mass= m2.mass - 10
        m2.radius = m2.mass
    if (s == 't'): 
        if plane.opacity == 0.5:
            plane.opacity = 0;
        else:
            plane.opacity=0.5
    if (s =='up'):
        tc = tc + 1
    if (s =='down'):
        tc = tc - 1

scene.bind('keydown', keyInput)

## initiate 
m1.mass = 500
m2.mass = 300
scene.range=30000
m1.radius = m1.mass # in units of solar r_s
m2.radius = m2.mass # in units of solar r_s

tc =  5*60 # 5 mins  to coalescence.  This is normally computed from the system clock, but that's ot available in glowscript
t = clock()
theta = 0
then = t
L = label(pos=vector(0.0,0.0,0.0),
    text='', yoffset = -100, line = False, color = vector(.8,.2,.2),
    height=120, box = False, opacity = 0,
    font='sans')
ocountdown = 0

while (tc-t)>0:
    Mc = ((m1.mass*m2.mass)**(3./5.) )/(m1.mass + m2.mass)**(1./5.) # chirp mass
    f = 0.5*( (8*pi)**(8./3.)/5. * (G*Mc*M_sun/c**3)**(5./3.) *(tc-t) )**(-3./8.) # orbital frequency
    r = (( G * M_sun*(m1.mass + m2.mass)/(2*pi*f)**2 )**(1./3.))/r_s # separation
 #   print( tc-t,f,r, m1.mass, m2.mass)
    r1 = r*m2.mass/(m1.mass + m2.mass)
    r2 = r*m1.mass/(m1.mass + m2.mass)
    now = clock()
    theta = theta + 2*pi*f*(now-then)
    then = now
    m1.pos = vector( -r1*cos(theta), -r1*sin(theta),0)
    m2.pos = vector( r2*cos(theta), r2*sin(theta),0)
    rate(100)
    t = now
    countdown = int(tc-t)
    if countdown != ocountdown:
        L.text = str(countdown)
        ocountdown = countdown


m1.opacity = 1    
m2.opacity = 0 
m1.radius = m1.mass+m2.mass

t0 = clock()
while clock() - t0 <60:
    rate(100)
    m1.pos = vector(0,0,1000*(clock() - t0))
   

